QTabBar::close-button {
  image: url(:/icons/closetab.svg);
}
QTabBar::close-button:hover {
  image: url(:/icons/closetab-hover.png);
}

#groupListToolbar {
  border: 0px;
}

#groupListToolbar::drop-down {
  border: 0px;
  background: transparent;
}

#groupListToolbar::down-arrow {
  image: url(:/icons/1downarrow.svg);
  width: 16px;
  height: 16px;
}
